﻿using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 그래프 헬퍼
    /// </summary>
    public static class GraphHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 너비 우선 탐색하기 - BreadthFirstSearch<T>(graph, startItem)

        /// <summary>
        /// 너비 우선 탐색하기
        /// </summary>
        /// <typeparam name="T">항목 타입</typeparam>
        /// <param name="graph">그래프</param>
        /// <param name="startItem">시작 항목</param>
        /// <returns>너비 우선 탐색 결과</returns>
        public static HashSet<T> BreadthFirstSearch<T>(Graph<T> graph, T startItem)
        {
            HashSet<T> visitedVertexHashSet = new HashSet<T>();

            if(!graph.AdjacencyDictionary.ContainsKey(startItem))
            {
                return visitedVertexHashSet;
            }
                
            Queue<T> queue = new Queue<T>();

            queue.Enqueue(startItem);

            while(queue.Count > 0)
            {
                T vertex = queue.Dequeue();

                if(visitedVertexHashSet.Contains(vertex))
                {
                    continue;
                }

                visitedVertexHashSet.Add(vertex);

                foreach(T neighborVertex in graph.AdjacencyDictionary[vertex])
                {
                    if(!visitedVertexHashSet.Contains(neighborVertex))
                    {
                        queue.Enqueue(neighborVertex);
                    }
                }
            }

            return visitedVertexHashSet;
        }

        #endregion
    }
}